#include <TTree.h>
#include <TFile.h>
#include <TCanvas.h>
#include <TApplication.h>
#include <TH2D.h>
#include <TF1.h>
#include <TGraphErrors.h>
#include <TSystem.h>
#include <TStyle.h>
#include <TPad.h>
#include <TMath.h>
#include <TLatex.h>

/* #include <stdlib.h> */
/* #include <cstdlib> */
/* #include <cmath> */
#include <fstream>
#include <iostream>
#include <stdio.h>

using namespace std;

#include "Math/Interpolator.h"
//#include "Math/Polynomial.h"
#include "Math/InterpolationTypes.h"

//#include "prm_reac.h" // parameter file <--
//#include "prm_reac_sim01.h" // parameter file <--
//#include "prm_reac_sim06.h" // parameter file <--
//#include "prm_reac_sim07.h" // parameter file <--
//#include "prm_reac_sim08.h" // parameter file <--
//#include "prm_reac_sim09.h" // parameter file <--
//#include "prm_scat.h" // parameter file <--
#include "../prm_reac.C" // parameter file <--

//Bool_t inv_thcm = true;
Bool_t inv_thcm = false;

//TFile *f = new TFile("si26ap0-0_250torr-480mm_49.4+-0.00MeV"
/* TFile *f = new TFile("si26ap0-0_250torr-480mm_49.4+-0.50MeV" */
/* 		     "_1.0ns_de520um_1e+06_ppacs300-680mm" */
/* 		     //"_1.0ns_de520um_1e+06_ppacs350-630mm" */
/* 		     "_tel-conf-01.root"); */
/* TFile *f = new TFile("si26ap0-3_250torr-480mm_49.4+-0.50MeV" */
/* 		     "_1.0ns_de520um_2e+06_ppacs300-680mm" */
/* 		     //"_1.0ns_de520um_1e+06_ppacs350-630mm" */
/* 		     "_tel-conf-01.root"); */
//TFile *f = new TFile("sim01.root");
TFile *f = new TFile(Form("../root/%s_%s.root",label,simnum));
//TFile *f = new TFile("tmp_t1_p1.root");
TTree *tr = (TTree*)f->Get("sim");

TF1 *fn1 = new TF1("fn1","gaus",-100,100);

extern "C" {
  /* void enewzsub_(int *z1, float *m1, float *bfr_ene, char matter1[33],  */
  /* 		 int *unit_pressure, float *pressure, float *temperature,  */
  /* 		 int *unit_thick, float *thick1, float *aft_ene); */
  /* void eoldzsub_(int *z1, float *m1, float *aft_ene, char matter1[33],  */
  /* 		 int *unit_pressure, float *pressure, float *temperature,  */
  /* 		 int *unit_thick, float *thick1, float *bfr_ene); */
  void rangezsub_(int *z1, float *m1, float *bfr_ene, char matter1[33], 
		  float *range);
  /* void srhogas_(float *pressure, float *temperature, int *unit_pressure, */
  /* 		int *unit_temperature, float *mol_weight,  */
  /* 		float *density); */
}

// effective thickness //
Double_t eff_thk(Double_t Ecm, Double_t dEcm){ // beam energy in GeV
  for(Int_t ichar=0;ichar<33;ichar++){ 
    if(he[ichar]=='\0'){
      he[ichar]=' ';
      he[ichar+1]='\0';
    }
  }
  float xpu;
  float range1, range2, drange;
  xpu = (Ecm + dEcm/2.)*(mprj + mtrg)/mprj/mtrg;
  rangezsub_(&zprj,&mprj,&xpu,he,&range1);
  xpu = (Ecm - dEcm/2.)*(mprj + mtrg)/mprj/mtrg;
  rangezsub_(&zprj,&mprj,&xpu,he,&range2);
  drange = fabs(range1 - range2);
  
  return(drange);
}



int main(){
  const Double_t pi = TMath::Pi();
  const Double_t lbsize = 0.05;
  //const Double_t txsize = 0.06;
  
  TApplication theApp("App", NULL, NULL);
  gStyle->SetTextFont(62);
  gStyle->SetTextSize(lbsize);
  
  gStyle->SetLabelFont(62,"XYZt");
  gStyle->SetLabelSize(lbsize,"XYZt");
  
  //gStyle->SetTitleFont(62);
  gStyle->SetTitleFont(62,"XYZt");
  gStyle->SetTitleSize(lbsize,"XYZt");
  gStyle->SetTitleFontSize(lbsize);
  gStyle->SetTitleYOffset(1.2);

  gStyle->SetPadTopMargin(0.06);
  gStyle->SetPadRightMargin(0.15);
  gStyle->SetPadLeftMargin(0.12);
  gStyle->SetPadBottomMargin(0.1);

  // gStyle->SetStatW(0.3);
  // gStyle->SetStatH(0.2);
  // gStyle->SetStatX(1);
  // gStyle->SetStatY(0.4);
  gStyle->SetOptStat(0);

  // Reading cross section data
  // NON-SMOKER data
  const Int_t nhf = 90; // no. of data
  Double_t ehf[nhf], notused, cshf[nhf];
  ifstream datahf("../../../../nonsmoker_cs.txt"); 
  for(Int_t i=0; i<nhf; i++ ){ // i=1 to avoid the 1st row
    datahf >> ehf[i] >> notused >> notused >> cshf[i];
    cshf[i] *= 1000./4./pi; // b -> mb/sr
  //cout << i << " " << ehf[i] <<" "<< cshf[i] << endl;
  }
  datahf.close();
  ROOT::Math::Interpolator interhf(nhf, ROOT::Math::Interpolation::kLINEAR);
  interhf.SetData(nhf, ehf, cshf); // MeV, mb
  
  // Canvas
  TCanvas *c1=new TCanvas("c1","c1",0,0,880,800);
  c1->Divide(3,3);
  //c1->Divide(2,2);
  c1->cd(1);
  
  // No. of injected beam particles
  //const Double_t Ibeam = 5.e4; // pps
  //const Double_t Ibeam = 1.e4; // pps
  const Double_t Ibeam = 2.e4; // pps
  const Double_t Dbeam = 5.; // days
  //const Double_t nbeam = 3.e10; // <=> 5e4 pps, 7 days
  const Double_t nbeam = Ibeam*Dbeam*24.*60.*60.;
  
  // No. of injected beam particles
  const Double_t Nav = 6.0221e23;
  const Double_t C_cm2_mb_mg = Nav/mtrg*1.e-30;
  //cout << C_cm2_mb_mg << endl;
  
  // Bin deffinition
  const Int_t necm = 140; // Ebin = 50 keV
  //const Int_t necm = 70; // Ebin = 100 keV
  const Double_t ecmmin = 0.;
  const Double_t ecmmax = 7.;
  const Double_t decm = (ecmmax - ecmmin)/double(necm);

  const Int_t nthcm = 90; // theta bin = 2 deg.
  const Double_t thcmmin = 0.; 
  const Double_t thcmmax = 180.;
  const Double_t dthcm = (thcmmax - thcmmin)/double(nthcm);
  const Double_t d2r = pi/180.;
  
  // Cut condition
  const char* cut =
    //"hit.idet==0&&iex==0"
    //"hit.idet>=0&&hit.idet<=2&&iex==0"
    "hit.any&&iex==0"
    //"&&pos.ztar>-236."; // to avoid events near the entrance
    "&&pos.ztar>-266."; // to avoid events near the entrance
    //    "&&(E.lab[0] < 32. || E.lab[0]>=32.&&(E.cm > 0.19*E.lab[0]+0.22))";
  Char_t hname[100];
  
  // TH2 for Nhit (Ecm vs thcm)
  sprintf(hname,"hhit");
  TH2D *hhit = new TH2D(hname,"Events hitting detectors;"
			"E_{c.m.} (MeV);#theta_{c.m.} (deg.)",
			necm,ecmmin,ecmmax,nthcm,thcmmin,thcmmax);  
  if(inv_thcm)
    tr->Draw(Form("180. - th.cm:E.cm>>%s",hname),cut);
  else
    tr->Draw(Form("th.cm:E.cm>>%s",hname),cut);
    
  // TH2 for Ntot (Ecm vs thcm)
  sprintf(hname,"htot");
  TH2D *htot = new TH2D(hname,"Total events;"
			"E_{c.m.} (MeV);#theta_{c.m.} (deg.)",
			necm,ecmmin,ecmmax,nthcm,thcmmin,thcmmax);  
  if(inv_thcm)
    tr->Draw(Form("180. - th.cm:E.cm>>%s",hname),"iex==0");
  else
    tr->Draw(Form("th.cm:E.cm>>%s",hname),"iex==0");
  
  sprintf(hname,"hsin_int");
  TH2D *hsin_int = new TH2D(hname,
			    "Integrated sin(#theta) in #theta bin size;"
			    "E_{c.m.} (MeV);#theta_{c.m.} (deg.)",
			    necm,ecmmin,ecmmax,nthcm,thcmmin,thcmmax);    
  
  // dO: TH2 Divide (Ecm vs thcm)
  TH2D *hdO = new TH2D("hdO","Solid angle (sr);"
		       "E_{c.m.} (MeV);#theta_{c.m.} (deg.)",
		       necm,ecmmin,ecmmax,nthcm,thcmmin,thcmmax);
  hdO->Divide(hhit,htot); //,4.*pi);
  /* hdO->Draw("colz"); */
  /* c1->Update(); gSystem->ProcessEvents(); getchar(); */
    
  // TH2 Fill: ds/dO: Diff. cross section vs Ecm (vs thcm)
  TH2D *hdsdO = new TH2D("hdsdO","Cross section (mb/sr);"
			 "E_{c.m.} (MeV);#theta_{c.m.} (deg.)",
  			 necm,ecmmin,ecmmax,nthcm,thcmmin,thcmmax);

  // TH2 Fill: thk/dE*dE: Eff. thick. vs Ecm
  TH2D *hthk = new TH2D("hthk",";"
			"E_{c.m.} (MeV);#theta_{c.m.} (deg.)",
  			necm,ecmmin,ecmmax,nthcm,thcmmin,thcmmax);
  
  Double_t ecm = ecmmin + decm/2.;
  for(Int_t i=0; i<necm; i++){

    Double_t w_cshf = interhf.Eval(ecm);
    Double_t w_efth = eff_thk(ecm,decm);

    Double_t thcm = thcmmin + dthcm/2.;
    for(Int_t j=0; j<nthcm; j++){
     hdsdO->Fill(ecm,thcm,w_cshf);
     hthk->Fill(ecm,thcm,w_efth);
     hsin_int->Fill(  ecm,thcm, - ( cos(d2r*(thcm+dthcm/2.))
				    - cos(d2r*(thcm-dthcm/2.)) )  );
     thcm += dthcm;      
    }
    ecm += decm;
  }

  hdO->Multiply(hdO,hsin_int,2.*pi);

  c1->cd(1);
  hhit->Draw("colz");
  c1->Update(); gSystem->ProcessEvents(); //getchar();

  c1->cd(2);
  htot->Draw("colz");
  c1->Update(); gSystem->ProcessEvents(); //getchar();

  c1->cd(3);
  hsin_int->Draw("colz");
  c1->Update(); gSystem->ProcessEvents(); //getchar();

  c1->cd(4);
  gPad->SetLogz(0);
  hdO->Draw("colz");
  c1->Update(); gSystem->ProcessEvents(); //getchar();
  
  c1->cd(5);
  gPad->SetLogz(1);
  hdsdO->GetZaxis()->SetRangeUser(1.e-8,1.e2);
  hdsdO->Draw("colz");
  c1->Update(); gSystem->ProcessEvents(); //getchar();

  c1->cd(6);
  gPad->SetLogz(0);
  hthk->SetTitle(Form("Effective thickness / %5.3f MeV (mg/cm^2)",decm));
  hthk->SetTitleFont(62);
  hthk->SetTitleSize(lbsize);
  hthk->Draw("colz");
  c1->Update(); gSystem->ProcessEvents(); //getchar();

  // Nbeam*ds/dO(mb/sr)*dO(sr)*thk/dE(mb/cm^2/MeV)*dE(MeV)*const(mg*mb/cm^2)
  c1->cd(8);
  TH2D *hyld = new TH2D("hyld",";E_{c.m.} (MeV);#theta_{c.m.} (deg.)",
  			necm,ecmmin,ecmmax,nthcm,thcmmin,thcmmax);
  gPad->SetLogz(1);
  hyld->Multiply(hdO,hdsdO);
  hyld->Multiply(hyld,hthk,nbeam*C_cm2_mb_mg);
  hyld->SetTitle(Form("Yield / %5.3f MeV / %3.1f deg.",decm,dthcm));
  hyld->SetTitleFont(62);
  hyld->SetTitleSize(lbsize);
  hyld->Draw("colz");
  c1->Update(); gSystem->ProcessEvents(); //getchar();

  /* TCanvas *c2=new TCanvas("c2","c2",1200,800,800,600); */
  c1->cd(7);
  TH1D *hdO_ecm = hdO->ProjectionX("hdO_ecm",0,nthcm);
  gPad->SetLogy(1);
  hdO_ecm->GetYaxis()->SetRangeUser(5.e-2,5.);
  hdO_ecm->Draw("HIST");
  c1->Update(); gSystem->ProcessEvents(); //getchar();

  c1->cd(9);
  TH1D *hyld_ecm = hyld->ProjectionX("hyld_ecm",0,nthcm);
  gPad->SetLogy(1);
  hyld_ecm->GetYaxis()->SetRangeUser(1.,1000.);
  hyld_ecm->SetTitle(Form("Yield / %5.3f MeV",decm));
  hyld_ecm->SetTitleFont(62);
  hyld_ecm->SetTitleSize(lbsize);
  hyld_ecm->Draw("HIST");
  Double_t ntot_yld = hyld_ecm->Integral(0,necm);
  //cout << hyld_ecm->Integral(0,necm) << endl;
  Double_t rate_yld = ntot_yld/Dbeam/24.; // counts/hour
  TLatex *t1=new TLatex(0.2,400,Form("Total: %.1f counts",ntot_yld));
  TLatex *t2=new TLatex(0.2,200,Form("Rate: %.1f counts/h",rate_yld));
  t1->Draw();
  t2->Draw();
  c1->Update(); gSystem->ProcessEvents();

  gSystem->Exec("mkdir -vp ../figs");//Creates figs/ directory if there is not
  //c1->Print(Form("../figs/ecm_yield_%s_tel1-3.png",label));
  c1->Print(Form("../figs/ecm_yield_%s_%s_telall.png",label,simnum));
  c1->Print(Form("../figs/ecm_yield_%s_%s_telall.pdf",label,simnum));
  

  /// To chech solid angle calculation
  /* c1->cd(5); */
  /* TH1D *htot_thcm = htot->ProjectionY("htot_thcm",0,necm); */
  /* htot_thcm->Draw(); */
  /* c1->Update(); gSystem->ProcessEvents(); getchar(); */
  
  /* c1->cd(8); */
  /* TH1D *htot_ecm = htot->ProjectionX("htot_ecm",0,nthcm); */
  /* htot_ecm->Draw(); */
  /* c1->Update(); gSystem->ProcessEvents(); getchar(); */
  
  /* c1->cd(6); */
  /* TH1D *hsin_int_thcm = hsin_int->ProjectionY("hsin_int_thcm",10,11); */
  /* hsin_int_thcm->Draw("HIST"); */
  /* c1->Update(); gSystem->ProcessEvents(); getchar(); */
    
  /* c1->cd(9); */
  /* TH1D *hsin_int_ecm = hsin_int->ProjectionX("hsin_int_ecm",0,nthcm); */
  /* hsin_int_ecm->Draw("HIST"); */
  /* c1->Update(); gSystem->ProcessEvents(); getchar(); */
  
  
  theApp.Run();
}
