/* Common constants */
const Double_t u = 931.49432/1000.; // GeV
const Double_t c = 299.7924; // speed of light, mm/ns 
const Double_t pi = 4.*atan(1.);
const Double_t d2r = pi/180.;
const Double_t dummy = -1000.;
//////////////////////////

/* Event information */
//Long64_t ntot = 1.e7;
Long64_t ntot = 2.e6;
//Long64_t ntot = 1.e5;
Char_t label[50] = "si26ap"; // 
Char_t simnum[10] = "sim000"; // No beam spread, detector resolutions (in energy, space)
//Char_t label[100] = "sim07"; // Geometry changed
//Char_t label[100] = "sim08"; // Geometry and beam energy changed
//Char_t label[100] = "sim09"; // PPAC b resolution
//Char_t label[100] = "sim10"; // PPAC b resolution
Char_t sor[2] = "p"; // (a,p) reaction
//////////////////////////


/* particles information */
Int_t z1 = 1;
float mh= 1.00782503;
Int_t z2 = 2;
float ma= 4.00260325;
Int_t z14 = 14;
float m26si = 25.99232992;
Int_t z15 = 15;
float m29p = 28.98180061;
/* const Int_t Nexci = 8; */
/* Double_t exci[Nexci] = {0., 1.383, 1.954, 2.423, // in 29P */
/* 		    3.106, 3.448, 4.080, 4.343}; */
const Int_t nex = 4;
Int_t iex;
Double_t Ex[nex] = {0., 1.383, 1.954, 2.423}; // in 29P
Int_t zprj = z14;
float mprj = m26si;       // massa proiettile: 26Si
Int_t ztrg = z2;
float mtrg = ma;       // massa target: alpha
Int_t zres = z15;
float mres[8]; //, q2[8];
float mres_gs = m29p;
Int_t zrec = z1;
float mrec = mh; 
Int_t atom_num[2] = {zres, zrec} ; // atomic numbers
void mres_q2(){
  for(Int_t i=0; i<nex; i++){ // define mass and q value for excited states
    mres[i] = mres_gs + Ex[i]/u/1000.; // (a,p) in MeV
    //q2[i] = (mprj + mtrg - mres[i] - mrec)*u*1000.; // in MeV
  }
}
//////////////////////////


/* parameters for eoldz */
float Eaft, Eaft_beam; 
float Ebfr, Ebfr_beam; // Beam energy E in MeV after energy loss in matter
Char_t he[34] = "he";
Char_t si[34] = "si";
Int_t unit_pressure = 1; // Torr
//float pressure0 = 250.;
//float pressure0 = 130.; //sim08
float pressure0 = 240.; //sim09 (do not input too much gas to avoid out of range error)
//float dpressure = 2.5; // sigma
float dpressure = 0.; // sigma
float pressure, temperature; // sigma
Int_t unit_temperature = 1; // K, for srho_gas
//float temperature0 = 293.;
float temperature0 = 299.;
//float dtemperature = 3.; // sigma
float dtemperature = 0.; // sigma
float density;
//float density = 5.4743e-05; // of the target gas (g/cm^3)
//Int_t unit_mgpcm2 = 2; // in mg/cm^2
Int_t unit_mm = 1; // in mm
//////////////////////////


/* Beam condition */
Double_t Ebeam_ini; // Beam energy at entrance in MeV
Double_t Ebeam_ini_ref = 57.2; // Reference beam energy at entrance in MeV
Double_t dEbeam = 1.8; // Beam energy spread (MeV)
Double_t depe2 = 0.01;
float ltar = 550.; // in mm // 250 torr, 49.4 MeV
Double_t za = -ltar/2. - 680.; // mm, PPAC a z position
Double_t zb = za + 350.; // mm, PPAC b z position
Double_t sgm_ppac_a = 0.5; // mm, PPAC a intrinsic resolution
Double_t sgm_ppac_b = 0.5; // mm, PPAC b intrinsic resolution
Double_t xtar0 = 2.85; // mm
Double_t ytar0 = 0.44; // mm 
Double_t xtar0_sgm = 5.0; // mm
Double_t ytar0_sgm = 5.8; // mm 
// Double_t xtar0 = 0.3; // mm sim10
// Double_t ytar0 = -1.63; // mm sim10
// Double_t xtar0_sgm = 3.3; // mm sim10
// Double_t ytar0_sgm = 8.8; // mm  sim10
//////////////////////////


// Detector arrangement
const Int_t ndet = 5; 
Double_t thdet[ndet] = {4., -27., 48., -62., 73.}; // in deg
Double_t ldet[ndet] = {240., 145., 115., 115., 110.};
Double_t xdet_off[ndet] = {0., 0., 0., 0., 0.};
Double_t zdet_off[ndet] = {20., 20., 20., -30., -80.};
float int_det = 10.; // interval between DE and E layers (mm)
//float de_thick = 0.02; // DE thickness (mm)
//float de_thick = 0.32; // DE thickness (mm)
float de_thick = 0.52; // DE thickness (mm)
Double_t psd_half[ndet] = {25., 25., 25., 25., 25.};
Int_t n_strip[ndet] = {16, 16, 16, 16, 16};
Double_t radius = ltar; // mm, for FindNextBoundary
Double_t dt = 1.; // TOF resolution in nsec.
//////////////////////////


// For Newton's Method
const Int_t nlim = 10; // Maximum number of trials
const Double_t dztar = .2; // dz in mm for defferentiation
const Double_t eps = 1.e-5; // 
