#include <TGeoManager.h>
#include <TGeometry.h>
#include <TNode.h>
#include <TGeoVolume.h>
#include <TGeoMatrix.h>
#include <TGeoMedium.h>
#include <TGeoMaterial.h>
#include <TGeoNode.h>
#include <TMaterial.h>
#include <TMixture.h>
#include <TShape.h>
#include <TString.h>
#include <Riostream.h>
#include <TFile.h>
#include <TCanvas.h>
#include <TH2F.h>
#include <TMath.h>
#include <TLorentzVector.h>
#include <TSystem.h>
#include <stdlib.h>
#include <TROOT.h>
#include <TGenPhaseSpace.h>
#include <TStyle.h>
#include "TGeoPatternFinder.h"
#include "TSystem.h"
#include <TGeoNavigator.h>
#include <iostream>
#include <cstdlib>
#include <fstream>
#include <cmath>
#include "TApplication.h"
#include "TF1.h"
#include "TGraph.h"
#include "TGraphErrors.h"
#include "TH1.h"
#include "TLegend.h"
#include "TLegendEntry.h"
#include "TColor.h"
#include <TRandom3.h>
#include <TTree.h>
#include <time.h> // for the seed of the dandom number
#include <TView3D.h>

#include "prm_reac.C" // parameter file <--

using namespace std;

/////////////////////////////////////////////////////////////////////////////
typedef struct{
  Double_t beam_dec, beam_ref, beam_ref_MC;
  Double_t lab[2], lab2_det, lab2_det_MC, lab2_MC;
  Double_t lab2_DE, lab2_DE_MC, lab2_RES, lab2_RES_MC;
  //Double_t rel, rel_MC, cm, cm_z, cm_z_ref, cm_MC, cm_z_MC; //, cm_dev;
  Double_t rel, rel_MC, cm, cm_ref, cm_MC;
} ENERGY;

typedef struct{
  Double_t lab[2], lab2_MC, cm, cm_MC; //, cm_dev, cm_MC_nr;
} THETA;

typedef struct {
  Double_t lab[2], lab2_MC; 
} PHI;

typedef struct {
  Double_t xtar, ytar, rtar, ztar;
  Double_t xtar_MC, ytar_MC, rtar_MC, ztar_MC;
  Double_t ldec_det, ldec_det_MC;
  Double_t xhit[ndet], yhit[ndet], xhit_MC[ndet], yhit_MC[ndet];  
} POSITION;

typedef struct {
  Double_t beam_ent_dec, beam_ent_dec_MC,
    rec_dec_det, rec_dec_det_MC,
    ent_det, ent_det_MC;
} TOF;

typedef struct{
  Int_t idet;
  Bool_t any;
  //Int_t any,idet;
} HIT;

// typedef struct{
//   Double_t prs, prs0, tmp, tmp0;
// } GAS;

/////////////////////////////////////////////////////////////////////////////
extern "C" {
  void enewzsub_etot_(int *z1, float *m1, float *bfr_ene, char matter1[33], 
		      int *unit_pressure, float *pressure, float *temperature, 
		      int *unit_thick, float *thick1, float *aft_ene);
  void eoldzsub_etot_(int *z1, float *m1, float *aft_ene, char matter1[33], 
		      int *unit_pressure, float *pressure, float *temperature, 
		      int *unit_thick, float *thick1, float *bfr_ene);
  void rangezsub_etot_(int *z1, float *m1, float *bfr_ene, char matter1[33], 
		       float *range);
  void srhogas_(float *pressure, float *temperature, int *unit_pressure,
  		int *unit_temperature, float *mol_weight, 
  		float *density);
}

/////////////////////////////////////////////////////////////////////////////
Double_t dtof(Double_t *x, Double_t *p){
  for(Int_t ichar=0;ichar<33;ichar++){ 
    if(he[ichar]=='\0'){
      he[ichar]=' ';
      he[ichar+1]='\0';
    }
  }

  int z = int(p[0]);
  float m = float(p[1]);
  float eb = float(p[2]);
  float l = float(x[0]);
  // to be integrated: dtof = dx/v(x)
  float ea; // MeV
  enewzsub_etot_(&z, &m, &eb, he, &unit_pressure, &pressure, 
	    &temperature, &unit_mm, &l, &ea);
  return sqrt(u*1000./2./ea*m)/c;
}

/////////////////////////////////////////////////////////////////////////////
Double_t f_s(Double_t s, Double_t plab, Double_t costhlab){  // GeV^2, GeV/c
  Double_t pcm2 = ( pow(s - u*mprj*u*mprj - u*mtrg*u*mtrg,2)
		    - 4.*u*mprj*u*mprj*u*mtrg*u*mtrg )/4./s;
  Double_t ppcm2 = ( pow(s - u*mres[iex]*u*mres[iex] - u*mrec*u*mrec,2)
		     - 4.*u*mres[iex]*u*mres[iex]*u*mrec*u*mrec )/4./s;  
  Double_t chi = log( (sqrt(pcm2) + sqrt(u*mtrg*u*mtrg + pcm2))/u/mtrg );
  Double_t tanhchi = tanh(chi);
  Double_t coshchi = cosh(chi);

  return 1.
    - costhlab*costhlab*tanhchi*tanhchi
    + u*mrec*u*mrec/plab/plab*tanhchi*tanhchi
    - 2.*sqrt(ppcm2 + u*mrec*u*mrec)/plab*costhlab*tanhchi/coshchi
    - ppcm2/plab/plab/coshchi/coshchi; // GeV^2/c^2
}

/////////////////////////////////////////////////////////////////////////////
//Double_t Elab2cm_rel(TLorentzVector LV, Int_t iex, // GeV/c
Double_t Elab2cm_rel(Double_t Elab, Double_t thlab, // GeV
		     Double_t Ecm_ini, Double_t dss, Int_t nlim, Double_t eps){ // GeV
  //Double_t plab = LV.P();
  Double_t plab = sqrt(pow(Elab,2) + 2.*Elab*u*mrec); // GeV/c
  Double_t s = pow(Ecm_ini + u*(mprj + mtrg),2); // GeV^2
  Double_t ds = dss*s;
  //Double_t costhlab = cos(LV.Theta());
  Double_t costhlab = cos(thlab*d2r);
  
  for(Int_t i=0; i<nlim; i++){
    // Standard at s
    Double_t fs = f_s(s, plab, costhlab); // GeV^2/c^2
    // Judge
    if(abs(fs) < eps){
      break;
    }
    // differential
    Double_t diff = (f_s(s+ds, plab, costhlab)
		     - f_s(s-ds, plab, costhlab))/2./ds;
    // Next s
    s -= fs/diff;
  }
  Double_t Ecm = sqrt(s) - u*(mprj + mtrg); // GeV
  return Ecm;
}


/////////////////////////////////////////////////////////////////////////////
Double_t ebeam2thk(Double_t Ebeam, Double_t Ebeam_ini){ //at z & entrance
  for(Int_t ichar=0;ichar<33;ichar++){ 
    if(he[ichar]=='\0'){
      he[ichar]=' ';
      he[ichar+1]='\0';
    }
  }
  
  float x;
  float range1, range2, drange;

  x = Ebeam; // MeV/u
  rangezsub_etot_(&zprj,&mprj,&x,he,&range1);
  x = Ebeam_ini;// MeV/u
  rangezsub_etot_(&zprj,&mprj,&x,he,&range2);

  drange = range2 - range1; // mg/cm^2
  
  return(drange);
}

/////////////////////////////////////////////////////////////////////////////
Double_t sgm_ppac_ab(Double_t z){
  Double_t sgm = sqrt(pow((zb-z)/(zb-za),2)*sgm_ppac_a
		      + pow((za-z)/(zb-za),2)*sgm_ppac_b);
  return sgm;
};

/////////////////////////////////////////////////////////////////////////////
Double_t theta_cm(Double_t mprj, Double_t mtrg, TLorentzVector LV_beam, TVector3 vel[3]){
  TVector3 vprj = pow(u*mprj,-1)/LV_beam.Gamma() *(LV_beam.Vect()); // velocity 
  /* TLorentzVector ptran = - LV3; // 4-momentum of the transferred particle  */
  /* TVector3 vtran = 1./sqrt(pow(mtrg*u - LV3.Energy(),2) + LV3.Mag2()) */
  /*   /ptran.Gamma() *ptran.Vect(); // velocity, See Slaus et al */
  TVector3 ki = vprj; // - vtran;
  TVector3 kf = vel[0] - vel[1]; // Final relative velocity (1<->2); vel[0] is beamlike (heavy ion)
  return (ki.Angle(kf))/d2r;  
}

/////////////////////////////////////////////////////////////////////////////
Double_t strip_position(Double_t x, Int_t n_strip, Double_t psd_half){
  Double_t x_strip = x;
  Double_t w_strip = psd_half*2./double(n_strip); // strip width
  Double_t x0 = - psd_half;
  Double_t x1 = x0 + w_strip;
  for(Int_t i=0; i<n_strip; i++){
    if(x > x0 && x <= x1){
      x_strip = (x0 + x1)/2.;
      break;
    }
    x0 = x1;
    x1 += w_strip; 
  }
  return x_strip;
}

/////////////////////////////////////////////////////////////////////////////
std::tuple<Double_t, Double_t, Double_t, TVector3>
  f_z(Double_t Ebeam, Double_t x, Double_t y, Double_t z,
      TVector3 vhit, TVector3 vdet){  // GeV^2, GeV/c
  Double_t Ecm_beam = Ebeam*mtrg/(mprj+mtrg);
  TVector3 vreac(x,y,z);
  TVector3 vdec_det = vhit + vdet - vreac;
  float fldec_det = float(vdec_det.Mag());
  eoldzsub_etot_(&zrec, &mrec, &Eaft, he, &unit_pressure, &pressure0, 
		 &temperature0, &unit_mm, &fldec_det, &Ebfr);
  Double_t Elab2 = Ebfr; 
  Double_t thlab2 = (vdec_det.Theta())/d2r;
  Double_t Ecm_rec = Elab2cm_rel(Elab2/1000., thlab2,
				 Ecm_beam/1000., 1.e-6, 10, 1.e-6)*1000.;
  
  return std::forward_as_tuple(2.*(Ecm_beam - Ecm_rec)/(Ecm_beam + Ecm_rec),
			       Elab2, thlab2, vdec_det);
}

/////////////////////////////////////////////////////////////////////////////
std::tuple<ENERGY, THETA, PHI, POSITION, TVector3>
NewtonsMethod4recpos(ENERGY E, THETA th, PHI phi, POSITION pos,
		     TVector3 vhit_MC, TVector3 vdet){
  //--- Inputs ---//
  // pos.*tar, E.lab2_det_MC, E.beam_ref, vhit_MC
  //--- Outputs ---//
  // E.beam_ref_MC: Beam energy
  // E.cm_MC = E.beam_ref_MC*mtrg/(mprj+mtrg): Ecm from z
  // E.lab2_MC: Lab. E of prt 2
  // th.lab2_MC = thlab2_MC: Lab. theta of prt 2
  // phi.lab2_MC = (vdec_det_MC.Phi())/d2r: Lab. phi of prt 2 
  // pos.ztar_MC: z of reaction point
  // pos.ldec_det_MC = vdec_det_MC.Mag(): distance btwn reaction point to detector
  Double_t f, fp, fm;
  TVector3 vreac_MC, vdec_det_MC;
  float z_beam_dec_MC;
  pos.ztar_MC = pos.ztar;
  Ebfr_beam = Ebeam_ini_ref;
  Eaft = E.lab2_det_MC; // MeV
  E.beam_ref_MC = E.beam_ref; // initial value
  
  for(Int_t i=0; i<nlim; i++){ // no. of trials for Newton's method
    // Ecm from recoil energy at z
    std::tie(f, E.lab2_MC, th.lab2_MC, vdec_det_MC)
      = f_z(E.beam_ref_MC, pos.xtar_MC,pos.ytar_MC,pos.ztar_MC,
	    vhit_MC, vdet);

    // Judge if solved 
    if(fabs(f) < eps){
      break;
    }
    
    // Ecm from recoil energy at z+dz
    z_beam_dec_MC = pos.ztar_MC+dztar + ltar/2.;
    enewzsub_etot_(&zprj, &mprj, &Ebfr_beam, he, &unit_pressure, &pressure0, 
		   &temperature0, &unit_mm, &z_beam_dec_MC, &Eaft_beam);
    std::tie(fp, E.lab2_MC, th.lab2_MC, vdec_det_MC)
      = f_z(Eaft_beam, pos.xtar_MC,pos.ytar_MC,pos.ztar_MC+dztar,
	    vhit_MC, vdet);
    
    // Ecm from recoil energy at z-dz
    z_beam_dec_MC = pos.ztar_MC-dztar + ltar/2.;
    enewzsub_etot_(&zprj, &mprj, &Ebfr_beam, he, &unit_pressure, &pressure0, 
		   &temperature0, &unit_mm, &z_beam_dec_MC, &Eaft_beam);
    std::tie(fm, E.lab2_MC, th.lab2_MC, vdec_det_MC)
      = f_z(Eaft_beam, pos.xtar_MC,pos.ytar_MC,pos.ztar_MC-dztar,
	    vhit_MC, vdet);
    
    // differential (f(z) = E_beam - E_rec)
    Double_t dfdz = (fp - fm)/2./dztar;
    pos.ztar_MC -= f/dfdz;	  	  
    
    // For Ebeam at next ztar_MC
    z_beam_dec_MC = pos.ztar_MC + ltar/2.;
    enewzsub_etot_(&zprj, &mprj, &Ebfr_beam, he, &unit_pressure, &pressure0, 
		   &temperature0, &unit_mm, &z_beam_dec_MC, &Eaft_beam);
    E.beam_ref_MC = Eaft_beam;
  } // for i	  
  //E.cm_z_MC = E.beam_ref_MC*mtrg/(mprj+mtrg);
  E.cm_MC = E.beam_ref_MC*mtrg/(mprj+mtrg);
  //E.cm_MC = (Ecm_MC_beam + Ecm_MC_rec)/2.;
  phi.lab2_MC = (vdec_det_MC.Phi())/d2r;
  pos.ldec_det_MC = vdec_det_MC.Mag();  
  return std::forward_as_tuple(E, th, phi, pos, vdec_det_MC);
}

/////////////////////////////////////////////////////////////////////////////
int main() {
  TApplication theApp("App", NULL, NULL);
  //////////////////////////////////////
  // For enewz
  for(Int_t ichar=0;ichar<33;ichar++){ 
    if(he[ichar]=='\0'){
      he[ichar]=' ';
      he[ichar+1]='\0';
    }
    if(si[ichar]=='\0'){
      si[ichar]=' ';
      si[ichar+1]='\0';
    }
  }
  srhogas_(&pressure0, &temperature0, &unit_pressure,
  	   &unit_temperature, &mtrg, 
  	   &density); // to get density of gas
  
  //////////////////////////////////////
  // Function for TOF integration
  TF1 *fint = new TF1("fint",dtof,0.,ltar,3);
  
  //////////////////////////////////////
  // Geometrical definitions
  // Constants: ltar, ndet, psd_half[ndet], *det_off[ndet], thdet[ndet]
  // Outputs: vdet, (voff), top 
  gStyle->SetPalette(1);
  gSystem->Load("libPhysics.so");
  gSystem->Load("libGeom.so");
  TGeoManager *sc = new TGeoManager("scattcham","Scattering Chamber");
  TGeoMaterial *matVacuum = new TGeoMaterial("Vacuum",0,0,0);
  TGeoMaterial *matSi = new TGeoMaterial("Si",28.086,14,2.321);
  TGeoMedium *Vacuum = new TGeoMedium("Vacuum",1,matVacuum);
  TGeoMedium *Si = new TGeoMedium("Root Material",2,matSi);
  TGeoVolume *top = sc->MakeSphere("TOP",Vacuum,0.,ltar/2.,0.,180.,0.,360.); //mm
  sc->SetTopVolume(top);
  TGeoVolume *psd[ndet];
  // (X,Y,Z,rotation angle theta originating at (X,Y,Z) )
  // tan(theta) = X/Z
  TGeoCombiTrans *posdet[ndet];
  TVector3 vdet[ndet], voff[ndet];
  // Detectors 
  for(Int_t idet=0;idet<ndet;idet++){
    psd[idet] = sc->MakeBox("PSD",Si,psd_half[idet],psd_half[idet],0.5); // in mm
    vdet[idet].SetXYZ( ldet[idet]*sin(d2r*thdet[idet])+xdet_off[idet],
  		    0,ldet[idet]*cos(d2r*thdet[idet])+zdet_off[idet] );
    posdet[idet] = new 
      // Rotate the detector
      TGeoCombiTrans(vdet[idet].X(),vdet[idet].Y(),vdet[idet].Z(),
      		     new TGeoRotation(Form("rot%d",idet),90,thdet[idet],0));
      // Or put the detectors at a coordinate 
      // TGeoCombiTrans(xdet[i],0.,zdet[i], new TGeoRotation(rotname,90,rotdet[i],0));
    top->AddNode(psd[idet],idet ,posdet[idet]);
    //voff[idet].SetXYZ(xdet_off[idet],0.,zdet_off[idet]);
  }  
  gGeoManager->CloseGeometry();
  gGeoManager->SetTopVisible();
  top->SetLineColor(kRed);
  TCanvas *c0 = new TCanvas("c0","c0",1410,10,800,800);
  top->Draw();
  TView3D *view = (TView3D*)TView::CreateView(1);
  view->SetRange(-ltar/2.,-ltar/2.,-ltar/2.,ltar/2.,ltar/2.,ltar/2.);
  view->Front();
  view->ShowAxis();    
  c0->Update(); 
  gSystem->ProcessEvents();
  //gSystem->Exec("mkdir -vp figs"); // Creates figs/ directory if there is not
  //c0->Print(Form("figs/det_arrange_%s_%s.png",label,simnum));
  
  //////////////////////////////////////
  // Output tree file
  gSystem->Exec("mkdir -vp root"); // Creates root/ directory if there is not
  Char_t fname[200]; 
  sprintf(fname,"root/%s_%s.root",label,simnum);
  cout << "Output file name: " << fname << endl;
  cout << "OK to run?" << endl; getchar();
  TFile *fl2 = new TFile(fname,"RECREATE");
  TTree* sim = new TTree("sim","");

  //////////////////////////////////////
  // Variables for tree
  ENERGY E;
  sim->Branch("E",&E,"beam_dec/D:beam_ref:beam_ref_MC:"
	      "lab[2]:lab2_det:lab2_det_MC:lab2_MC:"
	      "lab2_DE:lab2_DE_MC:lab2_RES:lab2_RES_MC:"
	      //"rel:rel_MC:cm:cm_z:cm_z_ref:cm_MC:cm_z_MC"); //:cm_dev");
	      "rel:rel_MC:cm:cm_ref:cm_MC");
  THETA th;
  sim->Branch("th",&th,"lab[2]/D:lab2_MC:cm:cm_MC"); //:cm_dev:cm_MC_nr");
  PHI phi;
  sim->Branch("phi",&phi,"lab[2]/D:lab2_MC");
  POSITION pos;
  sim->Branch("pos",&pos,
	      Form("xtar/D:ytar:rtar:ztar:"
		   "xtar_MC:ytar_MC:rtar_MC:ztar_MC:"
		   "ldec_det:ldec_det_MC:"
		   "xhit[%d]:yhit[%d]:"
		   "xhit_MC[%d]:yhit_MC[%d]",
		   ndet,ndet,ndet,ndet) );
  TOF tof;
  sim->Branch("tof",&tof,"beam_ent_dec/D:beam_ent_dec_MC:"
	      "rec_dec_det:rec_dec_det_MC:"
	      "ent_det:ent_det_MC");
  HIT hit;
  sim->Branch("hit",&hit,"idet/I:any/O");
  // GAS gas;
  // sim->Branch("gas",&gas,"prs/D:prs0:tmp:tmp0");
  //Int_t iex;
  sim->Branch("iex",&iex,"iex/I");  
  mres_q2(); // defines masses and Q values for iex
  
  ///////////////////////////////////////////////////////////////  
  // Kinematical quantities
  TVector3 vhit[ndet];
  TVector3 vhit_MC;
  TVector3 vdec_det, vdec_det_MC; // paticle #2
  TLorentzVector LV[2], LV_MC[2];
  TVector3 mom[2], mom_MC[2]; // particle #1, #2
  TVector3 pdir[2];
  TLorentzVector LVsum, LVsum_MC; // #1+#2
  TVector3 vel[3], vel_MC[3];
  TLorentzVector LV_target(0.0, 0.0, 0.0, u*mtrg); // px, py, pz, e
  TGenPhaseSpace event;
  
  ///////////////////////////////////////////////////////////////
  // Event loop
  Int_t seed = time(NULL);
  TRandom3* ran = new TRandom3(seed);
  Long64_t geve=0;
  Long64_t nstep = ntot/4321;
  for(Int_t ievt=0; ievt<ntot; ievt++){
    // to show the process //
    if(ievt%nstep == 0) 
      printf("%6.2f %% done, %6.2f %% hits detectors \r",
	     float(ievt)/float(ntot)*100.,
	     float(geve)/float(ievt)*100.);     

    // Initialization //
    E = {dummy, dummy, dummy,
	 dummy, dummy, dummy, dummy,
	 dummy, dummy, dummy, dummy,
	 dummy, dummy, dummy, dummy,
	 //dummy, dummy, dummy};
	 dummy}; 
    th = {0};
    phi = {0};
    tof = {dummy, dummy, dummy, dummy, dummy, dummy};
    hit.idet = -1;
    hit.any = false; // initialization
    pos = {dummy, dummy, dummy, dummy, dummy,
	   dummy, dummy, dummy, dummy, dummy,
	   dummy, dummy, dummy, dummy, dummy,
	   dummy, dummy, dummy, dummy, dummy,
	   dummy, dummy, dummy, dummy, dummy,
	   dummy, dummy, dummy, dummy, dummy};
    iex = -1;
    // gas = {0};
    
    // Giving the excited state in 29P
    iex = nex * ran->Rndm(); 
    Double_t mass[2] = { u*mres[iex], u*mrec} ; // masses of 29P,p
    
    // Defining the reaction point uniformly in Ebeam
    Ebeam_ini = ran->Gaus(Ebeam_ini_ref,dEbeam); // Beam energy at entrance with broadening
    E.beam_dec = ran->Uniform(0.,Ebeam_ini); // Giving random beam energy inside the gas
    //E.cm_z = E.beam_dec*mtrg/(mprj+mtrg); // From E.beam_dec energy to Ecm
    E.cm = E.beam_dec*mtrg/(mprj+mtrg); // From E.beam_dec energy to Ecm
    float z_beam_dec = ebeam2thk(E.beam_dec,Ebeam_ini)/density/1000.*10.; // z at E.beam_dec 
    Ebfr_beam = Ebeam_ini_ref; // Beam energy at entrance without broadening
    enewzsub_etot_(&zprj, &mprj, &Ebfr_beam, he, &unit_pressure, &pressure0, 
		   &temperature0, &unit_mm, &z_beam_dec, &Eaft_beam);
    E.beam_ref = Eaft_beam; // Beam energy at z_beam_dec without broadening
    //E.cm_z_ref = E.beam_ref*mtrg/(mprj+mtrg); // From E.beam_ref to Ecm
    E.cm_ref = E.beam_ref*mtrg/(mprj+mtrg); // From E.beam_ref to Ecm
    
    // Giving random beam position (X,Y)
    pos.xtar = ran->Gaus(xtar0,xtar0_sgm);
    pos.ytar = ran->Gaus(ytar0,ytar0_sgm);
    pos.ztar = z_beam_dec - ltar/2.; // Origin of z is entrance + ltar/2
    pos.rtar = sqrt(pos.xtar*pos.xtar + pos.ytar*pos.ytar);
    TVector3 vdec(pos.xtar,pos.ytar,pos.ztar); // vector to the reaction point.

    // Four-vector of the beam
    TLorentzVector LV_beam(0.0, 0.0,
			   sqrt(pow(E.beam_dec/1000.,2) + 2.*E.beam_dec*u*mprj/1000.),
			   u*mprj+E.beam_dec/1000.); // px, py, pz, e
    TLorentzVector LV_W = LV_beam + LV_target; // coumpound of beam + target
    event.SetDecay(LV_W, 2, mass); // W: beam + target -> 2-body decay: mres+mrec
    // w.evt =
    event.Generate(); // for 2-body decay, always 1 
    //cout << w.evt << endl;
    
    // Decay kinematics
    for(Int_t iprt=0; iprt<2; iprt++){
      LV[iprt] = *event.GetDecay(iprt); // lorentz vector      
      mom[iprt] = LV[iprt].Vect(); // momenta
      vel[iprt] = pow(mass[iprt],-1)/LV[iprt].Gamma() *mom[iprt]; // velocity
      th.lab[iprt] = (LV[iprt].Theta())/d2r; // lab theta
      phi.lab[iprt] = (LV[iprt].Phi())/d2r; // lab phi
      E.lab[iprt] = (LV[iprt].Energy()-mass[iprt])*1000.; // lab energy
    }
    // LVsum = LV[0] + LV[1];
    // E.rel = (LVsum.M() - (mass[0] + mass[1]))*1000.; // MeV
    // E.cm = E.rel - q2[iex]; // MeV
    th.cm = theta_cm(mprj, mtrg, LV_beam, vel);   
    
    /// Tracking Particle 2
    pdir[1] = mom[1].Unit(); // represents the direction
    Double_t xx2=pdir[1](0);
    Double_t yy2=pdir[1](1);
    Double_t zz2=pdir[1](2);
    gGeoManager->SetCurrentPoint(pos.xtar,pos.ytar,pos.ztar);
    gGeoManager->SetCurrentDirection(xx2,yy2,zz2);
    //      TGeoNode *current2 =
    gGeoManager->GetCurrentNode();
    gGeoManager->FindNode();
    gGeoManager->FindNextBoundary(radius);
    Double_t snext2 = gGeoManager->GetStep();
    //TGeoNode *newNode2 =
    gGeoManager->Step();
    Bool_t istate2 = gGeoManager->IsStepEntering();
    //Int_t idnode2 =
    gGeoManager->GetCurrentNodeId();
    //Double_t safety2 = gGeoManager->GetSafeDistance();
    const Char_t *path2 = gGeoManager->GetPath(); // /TOP_1/PSD_3 etc.
    Char_t sdet[2];
    strncpy(sdet, path2+11, 1); // extract the 12th character from path
    sdet[1] = '\0';
    vdec_det.SetXYZ(snext2*xx2,snext2*yy2,snext2*zz2);
    pos.ldec_det = snext2;
    
    if(istate2){ // When Particle 2 hits a detector
      hit.idet = atoi(sdet); // Integerization: no. of the hit detector // Warning: when no hit, it returns 0!
      hit.any = true;
      geve++;
      
      // Energy of Part. 2 at detector surface
      float fldec_det = float(pos.ldec_det);
      Ebfr = E.lab[1]; // MeV
      enewzsub_etot_(&zrec, &mrec, &Ebfr, he, &unit_pressure, &pressure, 
		     &temperature, &unit_mm, &fldec_det, &Eaft);
      E.lab2_det = Eaft; // MeV
      
      ///////////////////////////
      // Study with resolution //
      ///////////////////////////
      pressure = ran->Gaus(pressure0,dpressure);
      temperature = ran->Gaus(temperature0,dtemperature);
      // gas.prs0 = double(pressure0);
      // gas.prs = double(pressure);
      // gas.tmp0 = double(temperature0);
      // gas.tmp = double(temperature);

      // (X,Y) position with PPAC tracking resolution at the reaction point
      Double_t sgm_track = sgm_ppac_ab(pos.ztar); 
      pos.xtar_MC = ran->Gaus(pos.xtar,sgm_track);
      pos.ytar_MC = ran->Gaus(pos.ytar,sgm_track);
      pos.rtar_MC = sqrt(pos.xtar_MC*pos.xtar_MC + pos.ytar_MC*pos.ytar_MC);

      // (X,Y) hitting position on the detector plain
      // vhit to pos.*hit
      vhit[hit.idet] = vdec_det + vdec - vdet[hit.idet];
      vhit[hit.idet].RotateY(-d2r*thdet[hit.idet]);
      pos.xhit[hit.idet] = vhit[hit.idet].X();
      pos.yhit[hit.idet] = vhit[hit.idet].Y();

      // (X,Y) strip position on the detector plain
      // pos.*hit_MC to vhit_MC
      pos.xhit_MC[hit.idet]
	= strip_position( pos.xhit[hit.idet], n_strip[hit.idet], psd_half[hit.idet] );
      pos.yhit_MC[hit.idet]
	= strip_position( pos.yhit[hit.idet], n_strip[hit.idet], psd_half[hit.idet] );
      vhit_MC.SetXYZ( pos.xhit_MC[hit.idet], pos.yhit_MC[hit.idet], 0 ); //vector on det. surface
      vhit_MC.RotateY(d2r*thdet[hit.idet]); // Rotate from z axis to det. angle
      
      // Deviating Part. 2 energy with detector resolution
      E.lab2_det_MC = ran->Gaus(E.lab2_det, // based on resolution at 5 MeV
				sqrt(5.*E.lab2_det)*depe2); // MeV
      
      // Newton's method to search the reaction point (pos.ztar_MC) from
      //--- Inputs ---//
      // pos.*tar, E.lab2_det_MC, E.beam_ref, vhit_MC
      //if(!isnan( Elab2cm_rel(E.lab[1]/1000., th.lab[1], 
      //E.cm/1000., 1.e-6, 10, 1.e-6) )){
      std::tie(E, th, phi, pos, vdec_det_MC)
	= NewtonsMethod4recpos(E, th, phi, pos, vhit_MC, vdet[hit.idet]);
      //--- Outputs ---//
      // E.beam_ref_MC, E.cm_MC, E.lab2_MC,
      // th.lab2_MC, phi.lab2_MC, pos.ztar_MC, pos.ldec_det_MC
      //} // if
      float z_beam_dec_MC = pos.ztar_MC + ltar/2.;
      
      // Kinematics
      TLorentzVector LV_beam_MC(0.0, 0.0,
				sqrt(pow(E.beam_ref_MC/1000.,2) + 2.*E.beam_ref_MC*u*mprj/1000.),
				u*mprj+E.beam_ref_MC/1000.); // px, py, pz, e
      TLorentzVector LV_W_MC = LV_beam_MC + LV_target; // compound of beam + target
      LV_MC[1].SetE(mass[1] + E.lab2_MC/1000.);
      LV_MC[1].SetVect(vdec_det_MC.Unit()*sqrt(pow(E.lab2_MC/1000.,2) + 2.*E.lab2_MC*mass[1]/1000.));
      LV_MC[0] = LV_W_MC - LV_MC[1];	
      for(Int_t iprt=0; iprt<2; iprt++){
	mom_MC[iprt] = LV_MC[iprt].Vect(); // momenta
	vel_MC[iprt] = pow(mass[iprt],-1)/LV_MC[iprt].Gamma() *mom_MC[iprt]; // velocity
      }
      // LVsum_MC = LV_MC[0] + LV_MC[1];
      // E.rel_MC = (LVsum_MC.M() - (mass[0] + mass[1]))*1000.; // MeV
      // E.cm_MC = E.rel_MC - q2[iex]; // MeV
      th.cm_MC = theta_cm(mprj, mtrg, LV_beam_MC, vel_MC);   
      
      // TOF calucluations
      if( E.cm>1.5 && E.lab2_det>0. && E.lab2_det_MC>0.
	  // tentative E.cm limit to avoid GSL error 11
	  && !isnan(z_beam_dec) && !isnan(z_beam_dec_MC)
	  && !isnan(pos.ldec_det) && !isnan(pos.ldec_det_MC) ){
	
	// Beam particle
	fint->SetParameters(zprj,mprj,Ebeam_ini);
	tof.beam_ent_dec = fint->Integral(0.,z_beam_dec,1e-4);	  	  
	// Beam particle with resolution
	fint->SetParameters(zprj,mprj,Ebeam_ini_ref);
	tof.beam_ent_dec_MC = fint->Integral(0.,z_beam_dec_MC,1e-4);	
	// Recoil particle
	fint->SetParameters(zrec,mrec,E.lab[1]);
	tof.rec_dec_det = fint->Integral(0.,pos.ldec_det,1e-4);
	// Recoil particle with resolution
	fint->SetParameters(zrec,mrec,E.lab2_MC);
	tof.rec_dec_det_MC = fint->Integral(0.,pos.ldec_det_MC,1e-4);	
	// Total TOF
	tof.ent_det = tof.beam_ent_dec + tof.rec_dec_det; // TOF up to DE layer
	tof.ent_det_MC = ran->Gaus(tof.beam_ent_dec_MC + tof.rec_dec_det_MC,dt);
	
	// Energy deposit in DE layer 
	Ebfr = E.lab2_det; // Energy in front of DE (MeV)
	enewzsub_etot_(&zrec, &mrec, &Ebfr, si, &unit_pressure, &pressure, 
		       &temperature, &unit_mm, &de_thick, &Eaft);
	E.lab2_DE = Ebfr - Eaft; // Energy in DE
	E.lab2_DE_MC = ran->Gaus(E.lab2_DE, // based on resolution at 5 MeV
				 sqrt(5.*E.lab2_DE)*depe2);
	// Residual energy in E layer 
	Ebfr = Eaft; // Energy after DE
	enewzsub_etot_(&zrec, &mrec, &Ebfr, he, &unit_pressure, &pressure, 
		       &temperature, &unit_mm, &int_det, &Eaft);	
	E.lab2_RES = Eaft; // Energy before E
	E.lab2_RES_MC = ran->Gaus(E.lab2_RES, // based on resolution at 5 MeV
				  sqrt(5.*E.lab2_RES)*depe2);
      }// if(Elab2_det>0)      
    } // if istate2
    sim->Fill();	  
  } // for ievt
  fl2->Write();
  //fl2->Close();
  cout << "Number of events inside the detectors =" << geve << "\n"; 
  cout << "Total number of events =" << ntot << "\n"; 

  theApp.Run();
  
  return 0;
  
}


